@@ -165,8 +165,15 @@ pub trait ArchiveBuilder: Clone {
 /// Macro used by generator to define a flatdata struct.
 #[macro_export]
 macro_rules! define_struct {
+    // Simpler case where type and primitive_type coincide.
     ($name:ident, $name_mut:ident, $schema:expr, $size_in_bytes:expr
-        $(,($field:ident, $field_setter:ident, $type:tt, $offset:expr, $bit_size:expr))*) =>
+        $(,($field:ident, $field_setter:ident, $type:tt, $offset:expr, $bit_size:expr))*) => {
+        define_struct!($name, $name_mut, $schema, $size_in_bytes
+            $(,($field, $field_setter, $type: $type, $offset, $bit_size))*
+        );
+    };
+    ($name:ident, $name_mut:ident, $schema:expr, $size_in_bytes:expr
+        $(,($field:ident, $field_setter:ident, $type:tt: $primitive_type:tt, $offset:expr, $bit_size:expr))*) =>
     {
         // TODO: We cannot store `&u8` here, since then we need to annotate the type with a
         // lifetime, which would enforce an annotation in the trait, and this would bind the
@@ -180,7 +187,8 @@ macro_rules! define_struct {
 
         impl $name {
             $(pub fn $field(&self) -> $type {
-                read_bytes!($type, self.data, $offset, $bit_size)
+                let value = read_bytes!($primitive_type, self.data, $offset, $bit_size);
+                unsafe { ::std::mem::transmute::<$primitive_type, $type>(value) }
             })*
         }
 
@@ -188,7 +196,7 @@ macro_rules! define_struct {
             fn fmt(&self, f: &mut ::std::fmt::Formatter) -> ::std::fmt::Result {
                 write!(f,
                     concat!(stringify!($name), " {{ ",
-                        intersperse!($(concat!( stringify!($field), ": {}")), *), " }}"),
+                        intersperse!($(concat!( stringify!($field), ": {:?}")), *), " }}"),
                     $(self.$field(),)*)
             }
         }
@@ -222,7 +230,8 @@ macro_rules! define_struct {
 
         impl $name_mut {
             $(pub fn $field(&self) -> $type {
-                read_bytes!($type, self.data, $offset, $bit_size)
+                let value = read_bytes!($primitive_type, self.data, $offset, $bit_size);
+                unsafe { ::std::mem::transmute::<$primitive_type, $type>(value) }
             })*
 
             $(pub fn $field_setter(&mut self, value: $type) {
@@ -262,7 +271,7 @@ macro_rules! define_struct {
                 unsafe { &*(self as *const $name_mut as *const $name) }
             }
         }
-     }
+    };
 }
 
 /// Macro used by generator to define a flatdata index.
@@ -297,7 +306,7 @@ macro_rules! define_index {
 /// and `MultiArrayView`.
 #[macro_export]
 macro_rules! define_variadic_struct {
-    ($name:ident, $item_builder_name:ident, $index_type:tt,
+    ($name:ident, $item_builder_name:ident, $index_type:path,
         $($type_index:expr => ($type:tt, $add_type_fn:ident)),+) =>
     {
         #[derive(Clone, PartialEq)]
@@ -358,44 +367,83 @@ macro_rules! define_variadic_struct {
     }
 }
 
+/// Depending on the provided flag returns the type or wrap it in `Option`.
+///
+/// The flag is `true`, if the resource is optional.
+#[doc(hidden)]
+#[macro_export]
+macro_rules! opt {
+    ($type:ty, false) => {
+        $type
+    };
+    ($type:ty, true) => {
+        Option<$type>
+    };
+}
+
+/// Depending on whether the first argument is `true` or `false` returns the
+/// first block or the second, resp.
+#[doc(hidden)]
+#[macro_export]
+macro_rules! static_if {
+    (true, $true_block:block, $false_block:block) => {
+        $true_block
+    };
+    (false, $true_block:block, $false_block:block) => {
+        $false_block
+    };
+}
+
+/// Depending on the provided flag returns the result or make it an `Option`.
+///
+/// The flag is `true`, if the resource is optional.
+#[doc(hidden)]
+#[macro_export]
+macro_rules! check_resource {
+    ($res:expr,false) => {
+        $res?
+    };
+    ($res:expr,true) => {
+        $res.ok()
+    };
+}
+
 /// Macro used by generator to define a flatdata archive and corresponding
 /// archive builder.
 #[macro_export]
 macro_rules! define_archive {
     ($name:ident, $builder_name:ident, $archive_schema:expr;
         // struct resources
-        $(($struct_resource:ident,
-            $struct_setter:ident, $struct_type:tt, $struct_schema:expr)),*;
+        $(($struct_resource:ident, $struct_setter:ident, $struct_type:tt, $struct_schema:expr,
+            $is_optional_struct:ident)),*;
         // vector resources
-        $(($vector_resource:ident,
-            $vector_setter:ident, $vector_start:ident,
-            $element_type:tt, $element_schema:expr)),*;
+        $(($vector_resource:ident, $vector_setter:ident, $vector_start:ident,
+            $element_type:tt, $element_schema:expr, $is_optional_vector:ident)),*;
         // multivector resources
         $(($multivector_resource:ident,
             $multivector_start:ident,
             $variadic_type:tt, $variadic_type_schema:expr,
-            $multivector_resource_index:ident, $index_type:path)),*;
+            $multivector_resource_index:ident, $index_type:path,
+            $is_optional_multivector:ident)),*;
         // raw data resources
-        $(($raw_data_resource:ident,
-            $raw_data_resource_setter:ident, $raw_data_schema:expr)),*;
+        $(($raw_data_resource:ident, $raw_data_resource_setter:ident,
+            $raw_data_schema:expr, $is_optional_raw_data:ident)),*;
         // subarchive resources
         $(($subarchive_resource:ident,
-            $subarchive_type:tt, $subarchive_builder_type:tt, $subarchive_schema:expr)),*;
-        // optional subarchive resources
-        $(($opt_subarchive_resource:ident,
-            $opt_subarchive_type:tt, $opt_subarchive_builder_type:tt,
-            $opt_subarchive_schema:expr)),*
+            $subarchive_type:tt, $subarchive_builder_type:tt, $subarchive_schema:expr,
+            $is_optional_subarchive:ident)),*
     ) => {
 
         #[derive(Clone)]
         pub struct $name {
             _storage: ::std::rc::Rc<::std::cell::RefCell<$crate::ResourceStorage>>
-            $(,$struct_resource: $crate::MemoryDescriptor)*
-            $(,$vector_resource: $crate::MemoryDescriptor)*
-            $(,$multivector_resource: ($crate::MemoryDescriptor, $crate::MemoryDescriptor))*
-            $(,$raw_data_resource: $crate::MemoryDescriptor)*
-            $(,$subarchive_resource: $subarchive_type)*
-            $(,$opt_subarchive_resource: Option<$opt_subarchive_type>)*
+            $(,$struct_resource: opt!($crate::MemoryDescriptor, $is_optional_struct))*
+            $(,$vector_resource: opt!($crate::MemoryDescriptor, $is_optional_vector))*
+            $(,$multivector_resource: (
+                opt!($crate::MemoryDescriptor, $is_optional_multivector),
+                opt!($crate::MemoryDescriptor, $is_optional_multivector)))*
+            $(,$raw_data_resource: opt!($crate::MemoryDescriptor, $is_optional_raw_data))*
+            $(,$subarchive_resource: opt!($subarchive_type, $is_optional_subarchive))*
         }
 
         impl $name {
@@ -410,38 +458,73 @@ macro_rules! define_archive {
                 storage.read(name, schema).map(R::from)
             }
 
-            $(pub fn $struct_resource(&self) -> $crate::Handle<$struct_type> {
-                $crate::Handle::new($struct_type::from(self.$struct_resource.data()))
+            $(pub fn $struct_resource(&self) -> opt!(
+                $crate::Handle<$struct_type>, $is_optional_struct)
+            {
+                static_if!($is_optional_struct, {
+                    self.$struct_resource.as_ref().map(|mem_desc| {
+                        $crate::Handle::new($struct_type::from(mem_desc.data()))
+                    })
+                }, {
+                    $crate::Handle::new($struct_type::from(self.$struct_resource.data()))
+                })
             })*
 
-            $(pub fn $vector_resource(&self) -> $crate::ArrayView<$element_type> {
-                $crate::ArrayView::new(&self.$vector_resource)
+            $(pub fn $vector_resource(&self) -> opt!(
+                $crate::ArrayView<$element_type>, $is_optional_vector)
+            {
+                static_if!($is_optional_vector, {
+                    self.$vector_resource.as_ref().map($crate::ArrayView::new)
+                }, {
+                    $crate::ArrayView::new(&self.$vector_resource)
+                })
             })*
 
-            $(pub fn $multivector_resource(&self)
-                    -> $crate::MultiArrayView<$index_type, $variadic_type> {
-                $crate::MultiArrayView::new(
-                    $crate::ArrayView::new(&self.$multivector_resource.0),
-                    &self.$multivector_resource.1,
-                )
+            $(pub fn $multivector_resource(&self) -> opt!(
+                $crate::MultiArrayView<$index_type, $variadic_type>, $is_optional_multivector)
+            {
+                static_if!($is_optional_multivector, {
+                    let index_mem_desc = &self.$multivector_resource.0.as_ref();
+                    let res_mem_desc = &self.$multivector_resource.1.as_ref();
+                    index_mem_desc
+                        .map($crate::ArrayView::new)
+                        .and_then(move |index| {
+                            res_mem_desc.map(move |mem_desc| {
+                                $crate::MultiArrayView::new(index, mem_desc)
+                            })
+                        })
+                }, {
+                    $crate::MultiArrayView::new(
+                        $crate::ArrayView::new(&self.$multivector_resource.0),
+                        &self.$multivector_resource.1,
+                    )
+                })
             })*
 
-            $(pub fn $raw_data_resource(&self) -> &[u8] {
-                unsafe {
-                    ::std::slice::from_raw_parts(
-                        self.$raw_data_resource.data(),
-                        self.$raw_data_resource.size_in_bytes())
-                }
+            $(pub fn $raw_data_resource(&self) -> opt!(&[u8], $is_optional_raw_data) {
+                static_if!($is_optional_raw_data, {
+                    self.$raw_data_resource.as_ref().map(|mem_desc| {
+                        unsafe {
+                            ::std::slice::from_raw_parts(
+                                mem_desc.data(),
+                                mem_desc.size_in_bytes())
+                        }
+                    })
+                }, {
+                    unsafe {
+                        ::std::slice::from_raw_parts(
+                            self.$raw_data_resource.data(),
+                            self.$raw_data_resource.size_in_bytes())
+                    }
+                })
             })*
 
-            $(pub fn $subarchive_resource(&self) -> &$subarchive_type {
+            $(pub fn $subarchive_resource(&self) -> &opt!(
+                $subarchive_type, $is_optional_subarchive)
+            {
                 &self.$subarchive_resource
             })*
 
-            $(pub fn $opt_subarchive_resource(&self) -> &Option<$opt_subarchive_type> {
-                &self.$opt_subarchive_resource
-            })*
-
             fn signature_name(archive_name: &str) -> String {
                 format!("{}.archive", archive_name)
             }
@@ -457,7 +540,6 @@ macro_rules! define_archive {
                             $(, concat!(stringify!($multivector_resource), ": {:?}"))*
                             $(, concat!(stringify!($raw_data_resource), ": {:?}"))*
                             $(, concat!(stringify!($subarchive_resource), ": {:?}"))*
-                            $(, concat!(stringify!($opt_subarchive_resource), ": {:?}"))*
                         ),
                     " }}"),
                     $(self.$struct_resource(), )*
@@ -465,7 +547,6 @@ macro_rules! define_archive {
                     $(self.$multivector_resource(), )*
                     $(self.$raw_data_resource, )*
                     $(self.$subarchive_resource, )*
-                    $(self.$opt_subarchive_resource, )*
                 )
             }
         }
@@ -486,46 +567,47 @@ macro_rules! define_archive {
                     let res_storage = &mut *storage.borrow_mut();
                     res_storage.read(&Self::signature_name(Self::NAME), Self::SCHEMA)?;
 
-                    $($struct_resource = Self::read_resource(
-                        res_storage,
-                        stringify!($struct_resource),
-                        $struct_schema
-                    )?;
+                    $($struct_resource = check_resource!(
+                        Self::read_resource(
+                            res_storage,
+                            stringify!($struct_resource),
+                            $struct_schema
+                        ), $is_optional_struct);
                     )*
 
-                    $($vector_resource = Self::read_resource(
-                        res_storage,
-                        stringify!($vector_resource),
-                        $element_schema
-                    )?;
+                    $($vector_resource = check_resource!(
+                        Self::read_resource(
+                            res_storage,
+                            stringify!($vector_resource),
+                            $element_schema), $is_optional_vector);
                     )*
 
-                    $($multivector_resource_index = Self::read_resource(
-                        res_storage,
-                        stringify!($multivector_resource_index),
-                        &format!("index({})", $variadic_type_schema)
-                    )?;
-                    $multivector_resource = Self::read_resource(
-                        res_storage,
-                        stringify!($multivector_resource),
-                        $variadic_type_schema
-                    )?;
+                    $($multivector_resource_index = check_resource!(
+                        Self::read_resource(
+                            res_storage,
+                            stringify!($multivector_resource_index),
+                            &format!("index({})", $variadic_type_schema)),
+                        $is_optional_multivector);
+                    $multivector_resource = check_resource!(
+                        Self::read_resource(
+                            res_storage,
+                            stringify!($multivector_resource),
+                            $variadic_type_schema), $is_optional_multivector);
                     )*
 
-                    $($raw_data_resource = Self::read_resource(
-                        res_storage,
-                        stringify!($raw_data_resource),
-                        $raw_data_schema)?;
+                    $($raw_data_resource = check_resource!(
+                        Self::read_resource(
+                            res_storage,
+                            stringify!($raw_data_resource),
+                            $raw_data_schema), $is_optional_raw_data);
                     )*
                 }
                 $(
-                let $subarchive_resource = $subarchive_type::open(
-                    storage.borrow().subdir(&stringify!($subarchive_resource)))?;
-                )*
-                $(
-                let $opt_subarchive_resource = $opt_subarchive_type::open(
-                    storage.borrow().subdir(&stringify!($opt_subarchive_resource))).ok();
-                )*
+                let $subarchive_resource = check_resource!(
+                    $subarchive_type::open(
+                        storage.borrow().subdir(&stringify!($subarchive_resource))),
+                    $is_optional_subarchive
+                );)*
                 Ok(Self {
                     _storage: storage
                     $(,$struct_resource)*
@@ -535,7 +617,6 @@ macro_rules! define_archive {
                         $multivector_resource))*
                     $(,$raw_data_resource)*
                     $(,$subarchive_resource)*
-                    $(,$opt_subarchive_resource)*
                 })
             }
         }
@@ -605,15 +686,6 @@ macro_rules! define_archive {
                 $subarchive_builder_type::new(storage)
             }
             )*
-
-            $(pub fn $opt_subarchive_resource(
-                &mut self,
-            ) -> Result<$opt_subarchive_builder_type, $crate::ResourceStorageError> {
-                use $crate::ArchiveBuilder;
-                let storage = self.storage.borrow().subdir(stringify!($opt_subarchive_resource));
-                $opt_subarchive_builder_type::new(storage)
-            }
-            )*
         }
 
         impl $crate::ArchiveBuilder for $builder_name {
@@ -632,6 +704,7 @@ macro_rules! define_archive {
 
 #[cfg(test)]
 mod test {
+    use super::super::helper::Int;
     use super::super::structbuf::StructBuf;
 
     #[test]
@@ -649,4 +722,102 @@ mod test {
         let output = format!("{:?}", a);
         assert_eq!(output, "StructBuf { resource: A { x: 0, y: 0 } }");
     }
+
+    macro_rules! define_enum_test {
+        ($test_name:ident, $type:tt, $is_signed:expr, $val1:expr, $val2:expr) => {
+            #[test]
+            #[allow(dead_code)]
+            fn $test_name() {
+                #[derive(Debug, PartialEq, Eq)]
+                #[repr($type)]
+                pub enum Variant {
+                    X = $val1,
+                    Y = $val2,
+                }
+
+                impl Int for Variant {
+                    const IS_SIGNED: bool = $is_signed;
+                }
+
+                define_struct!(A, AMut, "no_schema", 1, (x, set_x, Variant: $type, 0, 2));
+                let mut a = StructBuf::<A>::new();
+                let output = format!("{:?}", a);
+                assert_eq!(output, "StructBuf { resource: A { x: X } }");
+
+                a.set_x(Variant::Y);
+                let output = format!("{:?}", a);
+                assert_eq!(output, "StructBuf { resource: A { x: Y } }");
+            }
+        };
+    }
+
+    define_enum_test!(test_enum_u8_1, u8, false, 0, 1);
+    define_enum_test!(test_enum_u8_2, u8, false, 0, 2);
+    define_enum_test!(test_enum_u16_1, u16, false, 0, 1);
+    define_enum_test!(test_enum_u16_2, u16, false, 0, 2);
+    define_enum_test!(test_enum_u32_1, u32, false, 0, 1);
+    define_enum_test!(test_enum_u32_2, u32, false, 0, 2);
+    define_enum_test!(test_enum_u64_1, u64, false, 0, 1);
+    define_enum_test!(test_enum_u64_2, u64, false, 0, 2);
+
+    // Note: Right now, there a regression bug for binary enums with underlying
+    // type i8: https://github.com/rust-lang/rust/issues/51582
+    //
+    // Until it is backported into stable release, we have to disable this test.
+    //
+    // define_enum_test!(test_enum_i8, i8, true, 0, 1);
+    // define_enum_test!(test_enum_i8, i8, true, 0, -1);
+    define_enum_test!(test_enum_i16_1, i16, true, 0, 1);
+    define_enum_test!(test_enum_i16_2, i16, true, 0, -1);
+    define_enum_test!(test_enum_i32_1, i32, true, 0, 1);
+    define_enum_test!(test_enum_i32_2, i32, true, 0, -1);
+    define_enum_test!(test_enum_i64_1, i64, true, 0, 1);
+    define_enum_test!(test_enum_i64_2, i64, true, 0, -1);
+
+    #[test]
+    #[allow(warnings)]
+    fn test_archive_compilation() {
+        // This test checks that the archive definition below compiles.
+        use super::Struct;
+
+        define_struct!(
+            A,
+            AMut,
+            "no_schema",
+            4,
+            (x, set_x, u32, 0, 16),
+            (y, set_y, u32, 16, 16)
+        );
+
+        define_index!(IndexType32, IndexType32Mut, "IndexType32 schema", 4, 32);
+
+        define_variadic_struct!(Ts, TsBuilder, IndexType32,
+            0 => (A, add_a));
+
+        define_archive!(SubArch, SubArchBuilder, "SubArch schema";
+            ;  // struct resources
+            ;  // vector resources
+            ;  // multivector resources
+            (raw, set_raw, "raw schema", false) // raw data resources
+            ; // subarchive resources
+        );
+
+        define_archive!(Arch, ArchBuilder, "Arch schema";
+            // struct resources
+            (a, set_a, A, "a schema", false),
+            (b, set_b, A, "b schema", true);
+            // vector resources
+            (v, set_v, start_v, A, "v schema", false),
+            (w, set_w, start_w, A, "w schema", true);
+            // multivector resources
+            (mv, start_mv, Ts, "mv schema", mv_index, IndexType32, false),
+            (mw, start_mw, Ts, "mw schema", mw_index, IndexType32, true);
+            // raw data resources
+            (r, set_r, "r schema", false),
+            (s, set_s, "s schema", true);
+            // subarchive resources
+            (arch, SubArch, SubArchBuilder, "arch schema", false),
+            (opt_arch, SubArch, SubArchBuilder, "opt_arch schema", true)
+        );
+    }
 }
